@@ -52,3 +52,25 @@ double getElasticNet(double grad, double hess, double a, double lam, double alph
   if (fabs(z) <= (lam * alpha)) return(0);
   else return(s * (fabs(z) - (lam * alpha)) / (hess + lam * (1 - alpha)));
 }
+
+//Include Group penalties (04/23/2020)
+double getGroupMcp(double z, double hess, double lam, double gamma) {
+  int s = sgn(z);
+  if (fabs(z) <= lam) return(0);
+  else if (fabs(z) <= gamma * lam) return(s * (fabs(z) - lam) / (hess * (1 - 1 / gamma)));
+  else return(z / hess);
+}
+
+double getGroupScad(double z, double hess, double lam, double gamma) {
+  int s = sgn(z);
+  if (fabs(z) <= lam) return(0);
+  else if (fabs(z) <= 2 * lam) return(s * (fabs(z) - lam) / hess);
+  else if (fabs(z) <= gamma * lam) return(s * (fabs(z) - gamma * lam / (gamma - 1)) / (hess * (1 - 1 / (gamma - 1))));
+  else return(z / hess);
+}
+
+double getGroupLasso(double z, double hess, double lam) {
+  int s = sgn(z);
+  if (fabs(z) <= lam) return(0);
+  else return(s * (fabs(z) - lam) / (hess));
+}
