@@ -2,7 +2,7 @@
 #define UTILS_H
 
 // This is the content of the .h file, which is where the declarations go
-double ridge(double z, double l1, double v);
+double getNorm(double *x, int p);
 double getMatrixVectorMultiplcation(double *X, double*y, int n, int j);
 double getWeightedCrossProduct(double *X, double *y, double *w, int n, int j);
 double getWeightedSumSquares(double *X, double *w, int n, int j);
